@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2008 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -387,8 +388,7 @@ free_themes (GSManager *manager)
 {
 	if (manager->priv->themes)
 	{
-		g_slist_foreach (manager->priv->themes, (GFunc)g_free, NULL);
-		g_slist_free (manager->priv->themes);
+		g_slist_free_full (manager->priv->themes, g_free);
 	}
 }
 
@@ -1077,8 +1077,10 @@ remove_unfade_idle (GSManager *manager)
 
 
 static gboolean
-window_deactivated_idle (GSManager *manager)
+window_deactivated_idle (gpointer data)
 {
+	GSManager *manager = data;
+
 	g_return_val_if_fail (manager != NULL, FALSE);
 	g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
@@ -1096,7 +1098,7 @@ window_deactivated_cb (GSWindow  *window,
 	g_return_if_fail (manager != NULL);
 	g_return_if_fail (GS_IS_MANAGER (manager));
 
-	g_idle_add ((GSourceFunc)window_deactivated_idle, manager);
+	g_idle_add (window_deactivated_idle, manager);
 }
 
 static GSWindow *
