@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -202,8 +203,7 @@ _gs_prefs_set_themes (GSPrefs *prefs,
 	guint i;
 	if (prefs->themes)
 	{
-		g_slist_foreach (prefs->themes, (GFunc)g_free, NULL);
-		g_slist_free (prefs->themes);
+		g_slist_free_full (prefs->themes, g_free);
 	}
 
 	/* take ownership of the list */
@@ -600,8 +600,7 @@ gs_prefs_finalize (GObject *object)
 
 	if (prefs->themes)
 	{
-		g_slist_foreach (prefs->themes, (GFunc)g_free, NULL);
-		g_slist_free (prefs->themes);
+		g_slist_free_full (prefs->themes, g_free);
 	}
 
 	g_free (prefs->logout_command);
