@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2008 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +44,7 @@
 
 static void gs_window_finalize   (GObject       *object);
 
-static gboolean popup_dialog_idle (GSWindow *window);
+static gboolean popup_dialog_idle (gpointer data);
 static void gs_window_dialog_finish (GSWindow *window);
 static void remove_command_watches (GSWindow *window);
 
@@ -380,7 +381,7 @@ gs_window_move_resize_window (GSWindow *window,
 	GdkWindow *gdkwindow;
 
 	widget = GTK_WIDGET (window);
-	gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
+	gdkwindow = gtk_widget_get_window (widget);
 
 	g_assert (gtk_widget_get_realized (widget));
 
@@ -636,7 +637,7 @@ remove_popup_dialog_idle (GSWindow *window)
 static void
 add_popup_dialog_idle (GSWindow *window)
 {
-	window->priv->popup_dialog_idle_id = g_idle_add ((GSourceFunc)popup_dialog_idle, window);
+	window->priv->popup_dialog_idle_id = g_idle_add (popup_dialog_idle, window);
 }
 
 static gboolean
@@ -1753,12 +1754,12 @@ popup_dialog (GSWindow *window)
 }
 
 static gboolean
-popup_dialog_idle (GSWindow *window)
+popup_dialog_idle (gpointer data)
 {
+	GSWindow *window = data;
+
 	popup_dialog (window);
-
 	window->priv->popup_dialog_idle_id = 0;
-
 	return FALSE;
 }
 
