@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -104,8 +105,9 @@ static void response_ok(void)
 	fflush(stdout);
 }
 
-static gboolean quit_response_ok(void)
+static gboolean quit_response_ok(gpointer data)
 {
+	(void)data;			/* remove warning unused parameter ‘data’ */
 	response_ok();
 	gtk_main_quit();
 	return FALSE;
@@ -320,8 +322,9 @@ static void response_cb(GSLockPlug* plug, gint response_id)
 	}
 }
 
-static gboolean response_request_quit(void)
+static gboolean response_request_quit(gpointer data)
 {
+	(void)data;			/* remove warning unused parameter ‘data’ */
 	printf("REQUEST QUIT\n");
 	fflush(stdout);
 	return FALSE;
@@ -333,8 +336,9 @@ static gboolean quit_timeout_cb(gpointer data)
 	return FALSE;
 }
 
-static gboolean auth_check_idle(GSLockPlug* plug)
+static gboolean auth_check_idle(gpointer data)
 {
+	GSLockPlug* plug = data;
 	gboolean res;
 	gboolean again;
 	static guint loop_counter = 0;
@@ -345,7 +349,7 @@ static gboolean auth_check_idle(GSLockPlug* plug)
 	if (res)
 	{
 		again = FALSE;
-		g_idle_add((GSourceFunc) quit_response_ok, NULL);
+		g_idle_add (quit_response_ok, NULL);
 	}
 	else
 	{
@@ -363,7 +367,7 @@ static gboolean auth_check_idle(GSLockPlug* plug)
 			/* Don't quit immediately, but rather request that mate-screensaver
 			 * terminates us after it has finished the dialog shake. Time out
 			 * after 5 seconds and quit anyway if this doesn't happen though */
-			g_idle_add((GSourceFunc) response_request_quit, NULL);
+			g_idle_add (response_request_quit, NULL);
 			g_timeout_add(5000, (GSourceFunc) quit_timeout_cb, NULL);
 		}
 	}
@@ -376,10 +380,11 @@ static void show_cb(GtkWidget* widget, gpointer data)
 	print_id(widget);
 }
 
-static gboolean popup_dialog_idle(void)
+static gboolean popup_dialog_idle(gpointer data)
 {
 	GtkWidget* widget;
 
+	(void)data;			/* remove warning unused parameter ‘data’ */
 	gs_profile_start(NULL);
 
 	widget = gs_lock_plug_new();
@@ -409,7 +414,7 @@ static gboolean popup_dialog_idle(void)
 
 	gtk_widget_realize(widget);
 
-	g_idle_add((GSourceFunc) auth_check_idle, widget);
+	g_idle_add(auth_check_idle, widget);
 
 	gs_profile_end(NULL);
 
@@ -425,7 +430,8 @@ static gboolean popup_dialog_idle(void)
    If the executable is setuid root, then these initializations
    are run as root, before discarding privileges.
 */
-static gboolean privileged_initialization(int* argc, char** argv, gboolean verbose)
+static gboolean
+privileged_initialization (int* argc, char** argv)
 {
 	gboolean ret;
 	char* nolock_reason;
@@ -467,7 +473,8 @@ static gboolean privileged_initialization(int* argc, char** argv, gboolean verbo
  *
  * Figure out what locking mechanisms are supported.
  */
-static gboolean lock_initialization (int* argc, char** argv, char** nolock_reason, gboolean verbose)
+static gboolean
+lock_initialization (int* argc, char** argv, char** nolock_reason)
 {
 	if (nolock_reason != NULL)
 	{
@@ -556,7 +563,7 @@ int main(int argc, char** argv)
 
 	gs_profile_start(NULL);
 
-	if (!privileged_initialization(&argc, argv, verbose))
+	if (!privileged_initialization (&argc, argv))
 	{
 		response_lock_init_failed();
 		exit(1);
@@ -581,7 +588,7 @@ int main(int argc, char** argv)
 		exit(1);
 	}
 
-	if (!lock_initialization(&argc, argv, &nolock_reason, verbose))
+	if (!lock_initialization(&argc, argv, &nolock_reason))
 	{
 		if (nolock_reason != NULL)
 		{
@@ -595,7 +602,7 @@ int main(int argc, char** argv)
 
 	gs_debug_init(verbose, FALSE);
 
-	g_idle_add((GSourceFunc) popup_dialog_idle, NULL);
+	g_idle_add (popup_dialog_idle, NULL);
 
 	gtk_main();
 
