@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,6 +39,7 @@
 
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #include <libmate-desktop/mate-desktop-utils.h>
+#include <libmate-desktop/mate-desktop-thumbnail.h>
 
 #include "gs-debug.h"
 
@@ -89,6 +91,7 @@ static GSJob          *job = NULL;
 static GSettings      *screensaver_settings = NULL;
 static GSettings      *session_settings = NULL;
 static GSettings      *lockdown_settings = NULL;
+static MateDesktopThumbnailFactory *thumb_factory = NULL;
 
 static gint32
 config_get_activate_delay (gboolean *is_writable)
@@ -190,7 +193,7 @@ config_get_theme (gboolean *is_writable)
 }
 
 static gchar **
-get_all_theme_ids (GSThemeManager *theme_manager)
+get_all_theme_ids (void)
 {
 	gchar **ids = NULL;
 	GSList *entries;
@@ -226,7 +229,7 @@ config_set_theme (const char *theme_id)
 		mode = GS_MODE_RANDOM;
 
 		/* set the themes key to contain all available screensavers */
-		strv = get_all_theme_ids (theme_manager);
+		strv = get_all_theme_ids ();
 	}
 	else
 	{
@@ -283,7 +286,7 @@ config_get_lock (gboolean *is_writable)
 }
 
 static gboolean
-config_get_lock_disabled ()
+config_get_lock_disabled (void)
 {
 	return g_settings_get_boolean (lockdown_settings, KEY_LOCK_DISABLE);
 }
@@ -295,8 +298,7 @@ config_set_lock (gboolean lock)
 }
 
 static void
-job_set_theme (GSJob      *job,
-               const char *theme)
+job_set_theme (const char *theme)
 {
 	GSThemeInfo *info;
 	const char  *command;
@@ -360,13 +362,13 @@ preview_set_theme (GtkWidget  *widget,
 	{
 		gchar **themes;
 
-		themes = get_all_theme_ids (theme_manager);
+		themes = get_all_theme_ids ();
 		if (themes != NULL)
 		{
 			gint32  i;
 
 			i = g_random_int_range (0, g_strv_length (themes));
-                        job_set_theme (job, themes[i]);
+                        job_set_theme (themes[i]);
                         g_strfreev (themes);
 
 			gs_job_start (job);
@@ -374,7 +376,7 @@ preview_set_theme (GtkWidget  *widget,
 	}
 	else
 	{
-		job_set_theme (job, theme);
+		job_set_theme (theme);
 		gs_job_start (job);
 	}
 }
@@ -807,8 +809,7 @@ theme_installer_run (GtkWidget *prefs_dialog, GList *files)
 	GtkWidget *copy_dialog;
 
 	copy_dialog = copy_theme_dialog_new (files);
-	g_list_foreach (files, (GFunc) (g_object_unref), NULL);
-	g_list_free (files);
+	g_list_free_full (files, g_object_unref);
 
 	gtk_window_set_transient_for (GTK_WINDOW (copy_dialog),
 	                              GTK_WINDOW (prefs_dialog));
@@ -1493,6 +1494,7 @@ widget_set_best_visual (GtkWidget *widget)
 static gboolean
 setup_treeview_idle (gpointer data)
 {
+	(void)data;			/* remove warning unused parameter ‘data’ */
 	GtkWidget *preview;
 	GtkWidget *treeview;
 
@@ -1520,6 +1522,75 @@ is_program_in_path (const char *program)
 	}
 }
 
+static void
+update_picture_filename_preview (GtkFileChooser *file_chooser,
+                                 gpointer        data)
+{
+	GtkWidget *preview;
+	char *uri;
+	gboolean have_preview = FALSE;
+
+	preview = GTK_WIDGET (data);
+	uri = gtk_file_chooser_get_preview_uri (file_chooser);
+	if (uri) {
+		GFile     *file;
+		GFileInfo *file_info;
+		time_t     mtime;
+		guint64    mtime_aux;
+		char      *thumb_path;
+
+		file = g_file_new_for_uri (uri);
+		file_info = g_file_query_info (file,
+		                               G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE","G_FILE_ATTRIBUTE_TIME_MODIFIED,
+		                               G_FILE_QUERY_INFO_NONE,
+		                               NULL, NULL);
+		g_object_unref (file);
+
+		if (file_info == NULL)
+			goto no_file_info;
+
+		mtime_aux = g_file_info_get_attribute_uint64 (file_info,
+		                                              G_FILE_ATTRIBUTE_TIME_MODIFIED);
+		mtime = (time_t)mtime_aux;
+		if ((thumb_path = mate_desktop_thumbnail_factory_lookup (thumb_factory, uri, mtime)) != NULL) {
+			/* try to load thumbnail from cache */
+			gtk_image_set_from_file (GTK_IMAGE (preview), thumb_path);
+			have_preview = TRUE;
+			g_free (thumb_path);
+		} else {
+			/* try to generate thumbnail from wallpaper & store to cache */
+			const char *content_type = g_file_info_get_content_type (file_info);
+			if (content_type != NULL) {
+				char *mime_type = g_content_type_get_mime_type (content_type);
+				if (mime_type != NULL) {
+					/* can generate thumbnail and did not fail previously */
+					if (mate_desktop_thumbnail_factory_can_thumbnail (thumb_factory, uri, mime_type, mtime)) {
+						GdkPixbuf *pixbuf;
+						pixbuf =  mate_desktop_thumbnail_factory_generate_thumbnail (thumb_factory,
+				                                                                             uri, mime_type);
+						if (pixbuf != NULL) {
+							gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
+							mate_desktop_thumbnail_factory_save_thumbnail (thumb_factory,
+							                                               pixbuf, uri, mtime);
+							have_preview = TRUE;
+							g_object_unref (pixbuf);
+						} else {
+							mate_desktop_thumbnail_factory_create_failed_thumbnail (thumb_factory,
+							                                                        uri, mtime);
+						}
+					}
+					g_free (mime_type);
+				}
+			}
+		}
+		g_object_unref (file_info);
+
+no_file_info:
+		g_free (uri);
+	}
+	gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
+}
+
 static void
 init_capplet (void)
 {
@@ -1541,6 +1612,7 @@ init_capplet (void)
 	GtkWidget *fullscreen_preview_next;
 	GtkWidget *fullscreen_preview_close;
 	GtkWidget *picture_filename;
+	GtkWidget *picture_filename_preview;
 	gdouble    activate_delay;
 	gboolean   enabled;
 	gboolean   is_writable;
@@ -1639,9 +1711,14 @@ init_capplet (void)
 	                  G_CALLBACK (lock_checkbox_toggled), NULL);
 
 	char *path = g_settings_get_string (screensaver_settings, "picture-filename");
-	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (gtk_builder_get_object (builder, "picture_filename")), path);
-	gtk_file_filter_add_pixbuf_formats (GTK_FILE_FILTER (gtk_builder_get_object (builder, "picture_filefilter")));
+	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (picture_filename), path);
 	g_free (path);
+	gtk_file_filter_add_pixbuf_formats (GTK_FILE_FILTER (gtk_builder_get_object (builder, "picture_filefilter")));
+	picture_filename_preview = gtk_image_new ();
+	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (picture_filename), picture_filename_preview);
+	thumb_factory = mate_desktop_thumbnail_factory_new (MATE_DESKTOP_THUMBNAIL_SIZE_LARGE);
+	g_signal_connect (picture_filename, "update-preview",
+	                  G_CALLBACK (update_picture_filename_preview), picture_filename_preview);
 	g_signal_connect (picture_filename, "selection-changed",
 	                  G_CALLBACK (picture_filename_changed), NULL);
 
@@ -1680,7 +1757,7 @@ init_capplet (void)
 	mode = g_settings_get_enum (screensaver_settings, KEY_MODE);
 	if (mode == GS_MODE_RANDOM) {
 		gchar **list;
-		list = get_all_theme_ids (theme_manager);
+		list = get_all_theme_ids ();
 		g_settings_set_strv (screensaver_settings, KEY_THEMES, (const gchar * const*) list);
 		g_strfreev (list);
 	}
@@ -1710,7 +1787,7 @@ init_capplet (void)
 	g_signal_connect (fullscreen_preview_next, "clicked",
 	                  G_CALLBACK (fullscreen_preview_next_cb), NULL);
 
-	g_idle_add ((GSourceFunc)setup_treeview_idle, NULL);
+	g_idle_add (setup_treeview_idle, NULL);
 }
 
 static void
@@ -1719,6 +1796,7 @@ finalize_capplet (void)
 	g_object_unref (screensaver_settings);
 	g_object_unref (session_settings);
 	g_object_unref (lockdown_settings);
+	g_object_unref (thumb_factory);
 }
 
 int
